/**
 * Simple example demonstrates communication with 2 U3s.
 *
 * support@labjack.com
 */

import java.io.*;
import java.text.*;
import com.sun.jna.*;
import com.sun.jna.ptr.*;
import com.labjack.LJUD;
import com.labjack.LJUDException;

public class TwoUnits {

	public TwoUnits() {
	}

	private void handleLJUDException(LJUDException e) {
		e.printStackTrace();
		if(e.getError() > LJUD.Errors.MIN_GROUP_ERROR.getValue()) {
			System.exit(-1);
		}
	}

	//Displays warning message if there is one. Error values < 0 are warnings
	//and do not cause a LJUDException in the LJUD class.
	private void checkForWarning(int error) {
		Pointer errorStringPtr = new Memory(256);
		if(error < 0) {
			LJUD.errorToString(error, errorStringPtr);
			System.out.println("Warning: " + errorStringPtr.getString(0).trim());
		}
	}

	public void runExample() {
		try {
			int intErrorcode = 0;
			IntByReference refIOType = new IntByReference(0);
			IntByReference refChannel = new IntByReference(0);
			DoubleByReference refValue = new DoubleByReference(0);
			IntByReference refHandle = new IntByReference(0);
			double value22 = 9999, value32 = 9999, value42 = 9999;
			double value23 = 9999, value33 = 9999, value43 = 9999;
			int intHandle2 = 0, intHandle3 = 0;
			IntByReference refHandle2 = new IntByReference(0);
			IntByReference refHandle3 = new IntByReference(0);
			
			String input = new String();
			boolean isDone = false;
	
			IntByReference dummyInt = new IntByReference(0);
			DoubleByReference dummyDouble = new DoubleByReference(0.0);
	
			String line = "";
			BufferedReader br = new BufferedReader(
					new InputStreamReader(System.in));
	
			//Read and display the UD version.
			System.out.println("UD Driver Version = " + LJUD.getDriverVersion());
	
			//Open the U3 with local ID 2.
			intErrorcode = LJUD.openLabJack(LJUD.Constants.dtU3, LJUD.Constants.ctUSB, "2", 0, refHandle2);
			checkForWarning(intErrorcode);
			intHandle2 = refHandle2.getValue();
			
			//Open the U3 with local ID 3.
			intErrorcode = LJUD.openLabJack(LJUD.Constants.dtU3, LJUD.Constants.ctUSB, "3", 0, refHandle3);
			checkForWarning(intErrorcode);
			intHandle3 = refHandle3.getValue();
	
			//Start by using the pin_configuration_reset IOType so that all
			//pin assignments are in the factory default condition.
			LJUD.ePut(intHandle2, LJUD.Constants.ioPIN_CONFIGURATION_RESET, 0, 0, 0);
			LJUD.ePut(intHandle3, LJUD.Constants.ioPIN_CONFIGURATION_RESET, 0, 0, 0);
	
			//First a configuration command.  These will be done with the ePut
			//function which combines the add/go/get into a single call.
	
			//Configure FIO2-FIO4 as analog, all else as digital, on both devices.
			//That means we will start from channel 0 and update all 16 flexible bits.
			//We will pass a value of b0000000000011100 or d28.
			LJUD.ePut(intHandle2, LJUD.Constants.ioPUT_ANALOG_ENABLE_PORT, 0, 28, 16);
			LJUD.ePut(intHandle3, LJUD.Constants.ioPUT_ANALOG_ENABLE_PORT, 0, 28, 16);
	
			//The following commands will use the add-go-get method to group
			//multiple requests into a single low-level function.
	
			//Request a single-ended reading from AIN2.
			LJUD.addRequest(intHandle2, LJUD.Constants.ioGET_AIN, 2, 0, 0, 0);
			LJUD.addRequest(intHandle3, LJUD.Constants.ioGET_AIN, 2, 0, 0, 0);
	
			//Request a single-ended reading from AIN3.
			LJUD.addRequest(intHandle2, LJUD.Constants.ioGET_AIN, 3, 0, 0, 0);
			LJUD.addRequest(intHandle3, LJUD.Constants.ioGET_AIN, 3, 0, 0, 0);
	
			//Request a reading from AIN4 using the Special 0-3.6 range.
			LJUD.addRequest(intHandle2, LJUD.Constants.ioGET_AIN_DIFF, 4, 0, 32, 0);
			LJUD.addRequest(intHandle3, LJUD.Constants.ioGET_AIN_DIFF, 4, 0, 32, 0);
	
			while(true) {
				//Execute all requests on all open LabJacks.
				LJUD.go();
	
				//Get all the results for unit 2.  The input measurement results are stored.
				LJUD.getFirstResult(intHandle2, refIOType, refChannel, refValue, dummyInt, dummyDouble);
	
				isDone = false;
				while(!isDone) {
					try {
						if(refIOType.getValue() == LJUD.Constants.ioGET_AIN) {
							switch(refChannel.getValue()) {
								case 2:
									value22 = refValue.getValue();
									break;
								case 3:
									value32 = refValue.getValue();
									break;
							}
						}
						else if(refIOType.getValue() == LJUD.Constants.ioGET_AIN_DIFF) {
							value42 = refValue.getValue();
						}
						LJUD.getNextResult(intHandle2, refIOType, refChannel, refValue, dummyInt, dummyDouble);
					}
					catch(LJUDException le) {
						if(le.getError() == LJUD.Errors.NO_MORE_DATA_AVAILABLE.getValue()) {
							isDone = true;
						}
						else {
							throw le;
						}
					}
				}
	
				//Get all the results for unit 3.  The input measurement results are stored.
				LJUD.getFirstResult(intHandle3, refIOType, refChannel, refValue, dummyInt, dummyDouble);
	
				isDone = false;
				while(!isDone) {
					try {
						if(refIOType.getValue() == LJUD.Constants.ioGET_AIN) {
							switch(refChannel.getValue()) {
								case 2:
									value23 = refValue.getValue();
									break;
								case 3:
									value33 = refValue.getValue();
									break;
							}
						}
						else if(refIOType.getValue() == LJUD.Constants.ioGET_AIN_DIFF) {
							value43 = refValue.getValue();
						}
						LJUD.getNextResult(intHandle3, refIOType, refChannel, refValue, dummyInt, dummyDouble);
					}
					catch(LJUDException le) {
						if(le.getError() == LJUD.Errors.NO_MORE_DATA_AVAILABLE.getValue()) {
							isDone = true;
						}
						else {
							throw le;
						}
					}
				}
	
				System.out.println("AIN2 (Unit 2) = " + value22);
				System.out.println("AIN2 (Unit 3) = " + value23);
				System.out.println("AIN3 (Unit 2) = " + value32);
				System.out.println("AIN3 (Unit 3) = " + value33);
				System.out.println("AIN4 (Unit 2) = " + value42);
				System.out.println("AIN4 (Unit 3) = " + value43);
		
				System.out.println("\nPress Enter to go again or (q) and Enter to quit");
				line = br.readLine().toUpperCase().trim();
				if(line.equals("Q")) {
					return;
				}
			}
		}
		catch(LJUDException le) {
			handleLJUDException(le);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static void main(String[] args) {
		new TwoUnits().runExample();
	}

}