/**
 *  Demonstrates the UD E-functions with the LabJack UE9.
 *
 *  support@labjack.com
 */

import com.sun.jna.*;
import com.sun.jna.ptr.*;
import com.labjack.LJUD;
import com.labjack.LJUDException;

public class EFunctions {

	public EFunctions() {
	}

	private void handleLJUDException(LJUDException e) {
		e.printStackTrace();
		if(e.getError() > LJUD.Errors.MIN_GROUP_ERROR.getValue()) {
			System.exit(-1);
		}
	}

	//Displays warning message if there is one. Error values < 0 are warnings
	//and do not cause a LJUDException in the LJUD class.
	private void checkForWarning(int error) {
		Pointer errorStringPtr = new Memory(256);
		if(error < 0) {
			LJUD.errorToString(error, errorStringPtr);
			System.out.println("Warning: " + errorStringPtr.getString(0).trim());
		}
	}
    
    public void runExample() {
    	try {
			int intErrorcode = 0;
			double dblValue = 0.0;
			DoubleByReference refDblValue = new DoubleByReference(0.0);
			int intValue = 0;
			IntByReference refIntValue = new IntByReference(0);
			int intHandle = 0;
			IntByReference refHandle = new IntByReference(0);
			int range = 0;
			int resolution = 0;
			int settling = 0;
			int binary = 0;
			int[] aEnableTimers = new int[6];
			int[] aEnableCounters = new int[2];
			int TCPinOffset, timerClockBaseIndex, timerClockDivisor;
			int[] aTimerModes = new int[6];
			double[] adblTimerValues = new double[6];
			int[] aReadTimers = new int[6];
			int[] aUpdateResetTimers = new int[6];
			int[] aReadCounters = new int[2];
			int[] aResetCounters = new int[2];
			double[] adblCounterValues = new double[2];
			double highTime, lowTime, dutyCycle;

			//Open the first found LabJack UE9.
			intErrorcode = LJUD.openLabJack(LJUD.Constants.dtUE9, LJUD.Constants.ctUSB, "1", 1, refHandle);
			//intErrorcode = LJUD.openLabJack(LJUD.Constants.dtUE9, LJUD.Constants.ctETHERNET, "192.168.1.209", 0, refHandle);
			checkForWarning(intErrorcode);
			intHandle = refHandle.getValue();
				
			//Take a measurement from AIN3.
			range = LJUD.Constants.rgBIP5V;
			resolution = 17;
			binary = 0;
			LJUD.eAIN(intHandle, 3, 0, refDblValue, range, resolution, 0, binary, 0, 0);
			System.out.println("AIN3 = " + refDblValue.getValue());
		
			//Set DAC0 to 3.0 volts.
			dblValue = 3.0;
			binary = 0;
			LJUD.eDAC(intHandle, 0, dblValue, binary, 0, 0);
			System.out.println("DAC0 set to " + dblValue + " volts");

			//Read state of FIO2.
			LJUD.eDI(intHandle, 2, refIntValue);
			System.out.println("FIO2 = " + refIntValue.getValue());

			//Set the state of FIO3.
			intValue = 0;
			LJUD.eDO(intHandle, 3, intValue);
			System.out.println("FIO3 set to = " + intValue + "\n");

			//Timers and Counters example.
			//First, a call to eTCConfig.  Fill the arrays with the desired values, then make the call.
			timerClockBaseIndex = LJUD.Constants.tc750KHZ;  //Choose 750 kHz base clock.
			timerClockDivisor = 3; //Divide by 3, thus timer clock is 250 kHz.
			aEnableTimers[0] = 1; //Enable Timer0 (uses FIO0).
			aEnableTimers[1] = 1; //Enable Timer1 (uses FIO1).
			aEnableTimers[2] = 1; //Enable Timer2 (uses FIO2).
			aEnableTimers[3] = 1; //Enable Timer3 (uses FIO3).
			aEnableTimers[4] = 0; //Disable Timer4.
			aEnableTimers[5] = 0; //Disable Timer5.
			aTimerModes[0] = LJUD.Constants.tmPWM8; //Timer0 is 8-bit PWM output.  Frequency is 250k/256 = 977 Hz.
			aTimerModes[1] = LJUD.Constants.tmDUTYCYCLE; //Timer1 is duty cyle input.
			aTimerModes[2] = LJUD.Constants.tmFIRMCOUNTER; //Timer2 is firmware counter input.
			aTimerModes[3] = LJUD.Constants.tmRISINGEDGES16; //Timer3 is 16-bit period measurement.
			aTimerModes[4] = 0; //Timer4 not enabled.
			aTimerModes[5] = 0; //Timer5 not enabled.
			adblTimerValues[0] = 16384; //Set PWM8 duty-cycle to 75%.
			adblTimerValues[1] = 0;
			adblTimerValues[2] = 0;
			adblTimerValues[3] = 0;
			adblTimerValues[4] = 0;
			adblTimerValues[5] = 0;
			aEnableCounters[0] = 1; //Enable Counter0 (uses FIO4).
			aEnableCounters[1] = 1; //Enable Counter1 (uses FIO5).
			LJUD.eTCConfig(intHandle, aEnableTimers, aEnableCounters, 0, timerClockBaseIndex, timerClockDivisor, aTimerModes, adblTimerValues, 0, 0);
			System.out.println("Timers and Counters enabled.\n");

			Thread.sleep(1000); //Wait 1 second.

			//Now, a call to eTCValues.
			aReadTimers[0] = 0; //Don't read Timer0 (output timer).
			aReadTimers[1] = 1; //Read Timer1;
			aReadTimers[2] = 1; //Read Timer2;
			aReadTimers[3] = 1; //Read Timer3;
			aReadTimers[4] = 0; //Timer4 not enabled.
			aReadTimers[5] = 0; //Timer5 not enabled.
			aUpdateResetTimers[0] = 1; //Update Timer0;
			aUpdateResetTimers[1] = 1; //Reset Timer1;
			aUpdateResetTimers[2] = 1; //Reset Timer2;
			aUpdateResetTimers[3] = 1; //Reset Timer3;
			aUpdateResetTimers[4] = 0; //Timer4 not enabled.
			aUpdateResetTimers[5] = 0; //Timer5 not enabled.
			aReadCounters[0] = 1; //Read Counter0;
			aReadCounters[1] = 1; //Read Counter1;
			aResetCounters[0] = 1; //Reset Counter0.
			aResetCounters[1] = 1; //Reset Counter1.
			adblTimerValues[0] = 32768; //Change Timer0 duty-cycle to 50%.
			adblTimerValues[1] = 0;
			adblTimerValues[2] = 0;
			adblTimerValues[3] = 0;
			adblTimerValues[4] = 0;
			adblTimerValues[5] = 0;
			LJUD.eTCValues(intHandle, aReadTimers, aUpdateResetTimers, aReadCounters, aResetCounters, adblTimerValues, adblCounterValues, 0, 0);
			System.out.println("Timer1 value = " + adblTimerValues[1]);
			System.out.println("Timer2 value = " + adblTimerValues[2]);
			System.out.println("Timer3 value = " + adblTimerValues[3]);
			System.out.println("Counter0 value = " + adblCounterValues[0]);
			System.out.println("Counter1 value = " + adblCounterValues[1]);

			//Convert Timer1 value to duty-cycle percentage.
			//High time is LSW
			highTime = (double)(((long)adblTimerValues[1]) % (65536));
			//Low time is MSW
			lowTime = (double)(((long)adblTimerValues[1]) / (65536));
			//Calculate the duty cycle percentage.
			dutyCycle = (( (highTime+lowTime) != 0)?(100*highTime/(highTime+lowTime)):0);
			System.out.println("\nHigh clicks Timer1 = " + highTime);
			System.out.println("Low clicks Timer1 = " + lowTime);
			System.out.println("Duty cycle Timer1 = " + dutyCycle);

			//Disable all timers and counters.
			aEnableTimers[0] = 0;
			aEnableTimers[1] = 0;
			aEnableTimers[2] = 0;
			aEnableTimers[3] = 0;
			aEnableTimers[4] = 0;
			aEnableTimers[5] = 0;
			aEnableCounters[0] = 0;
			aEnableCounters[1] = 0;
			LJUD.eTCConfig(intHandle, aEnableTimers, aEnableCounters, 0, timerClockBaseIndex, timerClockDivisor, aTimerModes, adblTimerValues, 0, 0);
		}
		catch(LJUDException le) {
			handleLJUDException(le);
		}
		catch (Exception e) {
			e.printStackTrace();
		}
    }
    
	public static void main(String[] args) {
		new EFunctions().runExample();
	}
}
