package com.labjack;

import com.sun.jna.*;
import com.sun.jna.ptr.*;


/**
 * LJUDException is a custom exception class for the {@link LJUD} class. It
 * takes the error code value returned from LJUD driver's function calls, will
 * get the string error from the LJUD driver and setup exception strings with
 * them.
 *
 * @author LabJack Corporation <support@labjack.com>
 */
public class LJUDException extends RuntimeException {
	private int error;
	private int errorAddress;
	private String errorString;
	private String message;
	
	public LJUDException(int error)
	{
		this.error = error;
		setErrorString();
		setMessage();
	}

	public int getError() {
		return error;
	}
	
	public String getErrorString() {
		return errorString;
	}
	
    @Override
    public String toString() {
    	return getClass().getName() + ": " + message;
    }

    @Override
    public String getMessage() {
    	return message;
    }
    
    //Sets errorString. Call after setting error.
    private void setErrorString() {
    	try {
			Pointer errorStringPtr = new Memory(256);
			LJUD.errorToString(error, errorStringPtr);
			errorString = errorStringPtr.getString(0).trim();
		}
		catch(Exception e) {
			//Error occured, just set to blank
			errorString = "";
		}
    }
    
    //Sets message. Call after setting error and errorString.
    private void setMessage() {
    	message =  "LJUD error " + error;
    	if(!errorString.isEmpty()) {
    		message += ". " + errorString.trim();
    	}
    }
}
